namespace GossamerVR.WearableAsset
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.CompilerServices;
    using UnityEngine;

    public static class PersonUtil
    {
        public static class Joint
        {
            public const string None = "None";
            public const string RootMale = "RootMale";
            public const string RootFemale = "RootFemale";
            public const string Head = "Head";
            public const string Neck = "Neck";
            public const string Chest = "Chest";
            public const string Abdomen = "Abdomen";
            public const string Pelvis = "Pelvis";
            public const string Hip = "Hip";

            public const string RShoulder = "R Shoulder";
            public const string RUpperArm = "R Upper Arm";
            public const string RForearm = "R Forearm";
            public const string RHand = "R Hand";
            public const string RThumb = "R Thumb";
            public const string RIndexF = "R Index F";
            public const string RMiddleF = "R Middle F";
            public const string RRingF = "R Ring F";
            public const string RPinkyF = "R Pinky F";

            public const string LShoulder = "L Shoulder";
            public const string LUpperArm = "L Upper Arm";
            public const string LForearm = "L Forearm";
            public const string LHand = "L Hand";
            public const string LThumb = "L Thumb";
            public const string LIndexF = "L Index F";
            public const string LMiddleF = "L Middle F";
            public const string LRingF = "L Ring F";
            public const string LPinkyF = "L Pinky F";

            public const string RThigh = "R Thigh";
            public const string RShin = "R Shin";
            public const string RFoot = "R Foot";

            public const string LThigh = "L Thigh";
            public const string LShin = "L Shin";
            public const string LFoot = "L Foot";

        }

        public static class Path
        {
            // the root path is from a person's atom transform.
            public const string RootFemale = "rescale2/PhysicsModel/Genesis2Female";
            public const string RootMale = "rescale2/PhysicsModel/Genesis2Male";

            // all other paths are relative to the root
            public const string Hip = "hip";
            public const string Pelvis = Hip + "/pelvis";
            public const string Abdomen = Hip + "/abdomen";
            public const string Chest = Abdomen + "/abdomen2/chest";
            public const string Neck = Chest + "/neck";
            public const string Head = Neck + "/head";

            public const string RShoulder = Chest + "/rCollar";
            public const string RUpperArm = RShoulder + "/rShldr";
            public const string RForearm = RUpperArm + "/rForeArm";
            public const string RHand = RForearm + "/rHand";
            public const string RThumb = RHand + "/rThumb1";
            public const string RIndexF = RHand + "/rCarpal1/rIndex1";
            public const string RMiddleF = RHand + "/rCarpal1/rMid1";
            public const string RRingF = RHand + "/rCarpal2/rRing1";
            public const string RPinkyF = RHand + "/rCarpal2/rPinky1";

            public const string LShoulder = Chest + "/lCollar";
            public const string LUpperArm = LShoulder + "/lShldr";
            public const string LForearm = LUpperArm + "/lForeArm";
            public const string LHand = LForearm + "/lHand";
            public const string LThumb = LHand + "/lThumb1";
            public const string LIndexF = LHand + "/lCarpal1/lIndex1";
            public const string LMiddleF = LHand + "/lCarpal1/lMid1";
            public const string LRingF = LHand + "/lCarpal2/lRing1";
            public const string LPinkyF = LHand + "/lCarpal2/lPinky1";

            public const string RThigh = Hip + "/pelvis/rThigh";
            public const string RShin = RThigh + "/rShin";
            public const string RFoot = RShin + "/rFoot";

            public const string LThigh = Hip + "/pelvis/lThigh";
            public const string LShin = LThigh + "/lShin";
            public const string LFoot = LShin + "/lFoot";

        }


        private static Dictionary<string, string> PathTable = new Dictionary<string, string>()
        {
            {Joint.RootMale, Path.RootMale},
            {Joint.RootFemale, Path.RootFemale },
            {Joint.Hip, Path.Hip },
            {Joint.Abdomen, Path.Abdomen },
            {Joint.Pelvis, Path.Pelvis },
            {Joint.Chest, Path.Chest },
            {Joint.Neck, Path.Neck },
            {Joint.Head, Path.Head },

            {Joint.RShoulder, Path.RShoulder },
            {Joint.RUpperArm, Path.RUpperArm},
            {Joint.RForearm, Path.RForearm },
            {Joint.RHand, Path.RHand },
            {Joint.RThumb, Path.RThumb },
            {Joint.RIndexF, Path.RIndexF },
            {Joint.RMiddleF, Path.RMiddleF },
            {Joint.RRingF, Path.RRingF },
            {Joint.RPinkyF, Path.RPinkyF },

            {Joint.LShoulder, Path.LShoulder },
            {Joint.LUpperArm, Path.LUpperArm},
            {Joint.LForearm, Path.LForearm },
            {Joint.LHand, Path.LHand },
            {Joint.LThumb, Path.LThumb },
            {Joint.LIndexF, Path.LIndexF },
            {Joint.LMiddleF, Path.LMiddleF },
            {Joint.LRingF, Path.LRingF },
            {Joint.LPinkyF, Path.LPinkyF },

            {Joint.RThigh, Path.RThigh },
            {Joint.RShin, Path.RShin},
            {Joint.RFoot, Path.RFoot },

            {Joint.LThigh, Path.LThigh },
            {Joint.LShin, Path.LShin },
            {Joint.LFoot, Path.LFoot },
        };

        public static string GetPath(string joint)
        {
            string ret_path = "";
            
            if (!PathTable.TryGetValue(joint, out ret_path))
            {
                SuperController.LogError($"PersonUtil.GetPath() has no path for joint: '{joint}'");
            }            
            return ret_path;
        }
        
        public static Transform FindRoot(Atom person)
        {
            Transform ret_root = null;
            string path = GetPath(Joint.RootFemale);
            if (person != null && !string.IsNullOrEmpty(path))
            {
                ret_root =person.transform.Find(path);
                if (ret_root == null)
                {
                    path = GetPath(Joint.RootMale);
                    ret_root = person.transform.Find(path);
                }
            }
            return ret_root;
        }

        public static Transform FindJoint(Transform root, string joint)
        {
            Transform ret_joint = null;
            string path = GetPath(joint);
            if (root != null && !string.IsNullOrEmpty(path))
            {
                ret_joint = root.Find(path);
            }
            return ret_joint;
        }

    }
}